"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _stepfunctions = _interopRequireDefault(require("aws-sdk/clients/stepfunctions"));

var _resolveDebugLevels = _interopRequireDefault(require("resolve-debug-levels"));

var _resolveSaga = require("resolve-saga");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const getLog = name => (0, _resolveDebugLevels.default)(`resolve:cloud:scheduler:${name}`);

const stateMachineArn = () => process.env['RESOLVE_CLOUD_SCHEDULER_STEP_FUNCTION_ARN'];

const STOP_ERROR_CODE = 'No error';
const STOP_ERROR_CAUSE = 'Scheduler stopped by user';
const EXECUTION_LIST_PAGE_SIZE = 100;

const start = async entry => {
  const log = getLog(`step-functions-start`);

  try {
    log.verbose(`entry: ${JSON.stringify(entry)}`);
    log.debug(`starting new execution ${entry.taskId}`);
    const sf = new _stepfunctions.default();
    await sf.startExecution({
      stateMachineArn: stateMachineArn(),
      name: entry.taskId,
      input: JSON.stringify({
        date: new Date(entry.date).toISOString(),
        event: {
          resolveSource: 'Scheduler',
          entry
        }
      })
    }).promise();
    log.debug('new execution started successfully');
  } catch (error) {
    if (error.code !== 'ExecutionAlreadyExists') {
      log.error(error.message);
      throw error;
    }
  }
};

const stopAll = async () => {
  const processPage = async (token = {}) => {
    const sf = new _stepfunctions.default();
    const {
      executions,
      nextToken
    } = await sf.listExecutions(Object.assign({
      stateMachineArn: stateMachineArn(),
      maxResults: EXECUTION_LIST_PAGE_SIZE,
      statusFilter: 'RUNNING'
    }, token)).promise();
    await Promise.all(executions.map(({
      executionArn
    }) => sf.stopExecution({
      executionArn,
      cause: STOP_ERROR_CAUSE,
      error: STOP_ERROR_CODE
    }).promise()));
    if (nextToken) await processPage({
      nextToken
    });
  };

  return processPage();
};

const errorHandler = async error => {
  throw error;
};

const isEmpty = obj => Object.keys(obj).reduce((empty, key) => empty && !obj.hasOwnProperty(key), true);

const validateEntry = ({
  date,
  taskId,
  command
}) => date != null && date.constructor === Number && taskId != null && taskId.constructor === String && command != null && command.constructor === Object && !isEmpty(command);

const initScheduler = resolve => {
  getLog('createAdapter').debug(`building new resolve cloud scheduler adapter`);
  resolve.scheduler = {
    async addEntries(data) {
      const log = getLog('addEntries');
      log.debug(`adding new scheduled entries`);
      log.verbose(`data: ${JSON.stringify(data)}`);
      const entries = [].concat(data);

      try {
        log.debug(`starting step function executions`);
        await Promise.all(entries.map(entry => validateEntry(entry) ? start(entry) : errorHandler(Error(`invalid entry ${JSON.stringify(entry)}`))));
        log.debug(`entries were successfully added`);
      } catch (e) {
        log.error(e.message);
        await errorHandler(e);
      }
    },

    async clearEntries() {
      const log = getLog('clearEntries');
      log.debug(`step functions cannot be recreated, skipping clearing`);
      await stopAll();
    },

    async executeEntries(data) {
      const log = getLog('executingEntries');
      log.debug(`executing scheduled entries`);
      log.verbose(`data: ${JSON.stringify(data)}`);
      const entries = [].concat(data);

      try {
        log.debug(`executing tasks`);
        await Promise.all(entries.map(({
          taskId,
          date,
          command
        }) => resolve.executeCommand({
          aggregateName: _resolveSaga.schedulerName,
          aggregateId: taskId,
          type: 'execute',
          payload: {
            date,
            command
          }
        })));
        log.debug(`tasks were successfully executed`);
      } catch (e) {
        log.error(e.message);
        await errorHandler(e);
      }
    }

  };
};

var _default = initScheduler;
exports.default = _default;
//# sourceMappingURL=init-scheduler.js.map