"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _resolveDebugLevels = _interopRequireDefault(require("resolve-debug-levels"));

var _lambda = require("resolve-cloud-common/lambda");

var _apiGatewayHandler = _interopRequireDefault(require("./api-gateway-handler"));

var _deployServiceEventHandler = _interopRequireDefault(require("./deploy-service-event-handler"));

var _schedulerEventHandler = _interopRequireDefault(require("./scheduler-event-handler"));

var _initScheduler = _interopRequireDefault(require("./init-scheduler"));

var _initMonitoring = _interopRequireDefault(require("./init-monitoring"));

var _metrics = require("./metrics");

var _initResolve = _interopRequireDefault(require("../common/init-resolve"));

var _disposeResolve = _interopRequireDefault(require("../common/dispose-resolve"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const log = (0, _resolveDebugLevels.default)('resolve:resolve-runtime:cloud-entry');
const GRACEFUL_WORKER_SHUTDOWN_TIME = 30 * 1000;

const getVacantTimeInMillis = lambdaContext => lambdaContext.getRemainingTimeInMillis() - GRACEFUL_WORKER_SHUTDOWN_TIME;

let coldStart = true;

const lambdaWorker = async (resolveBase, lambdaEvent, lambdaContext) => {
  log.debug('executing application lambda');
  log.verbose('incoming event', JSON.stringify(lambdaEvent, null, 2));
  lambdaContext.callbackWaitsForEmptyEventLoop = false;
  resolveBase.eventSubscriberCredentials = {
    mode: 'internal',
    applicationLambdaArn: lambdaContext.invokedFunctionArn,
    lambdaEventType: 'EventBus',
    lambdaEventName: 'resolveSource'
  };
  resolveBase.eventstoreCredentials = {
    mode: 'internal',
    applicationLambdaArn: lambdaContext.invokedFunctionArn,
    lambdaEventType: 'EventStore',
    lambdaEventName: 'resolveSource'
  };
  resolveBase.subscriptionsCredentials = {
    applicationLambdaArn: lambdaContext.invokedFunctionArn
  };

  resolveBase.invokeEventBusAsync = async (eventSubscriber, method, parameters) => {
    await (0, _lambda.invokeFunction)({
      FunctionName: lambdaContext.invokedFunctionArn,
      InvocationType: 'Event',
      Region: process.env.AWS_REGION,
      Payload: {
        resolveSource: 'EventBusDirect',
        method,
        payload: {
          eventSubscriber,
          ...parameters
        }
      }
    });
  };

  (0, _initMonitoring.default)(resolveBase);
  const resolve = Object.create(resolveBase);
  resolve.getVacantTimeInMillis = getVacantTimeInMillis.bind(null, lambdaContext);
  await (0, _initScheduler.default)(resolve);
  const lambdaRemainingTimeStart = lambdaContext.getRemainingTimeInMillis();

  try {
    log.debug('initializing reSolve framework');
    await (0, _initResolve.default)(resolve);
    log.debug('reSolve framework initialized');

    if (lambdaEvent.resolveSource === 'DeployService') {
      log.debug('identified event source: deployment service');
      const executorResult = await (0, _deployServiceEventHandler.default)(lambdaEvent, resolve);
      log.verbose(`executorResult: ${JSON.stringify(executorResult)}`);
      return executorResult;
    } else if (lambdaEvent.resolveSource === 'EventBusDirect') {
      log.debug('identified event source: event-bus-direct');
      const {
        method,
        payload
      } = lambdaEvent;
      const executorResult = await resolve.eventBus[method](payload);
      log.verbose(`executorResult: ${JSON.stringify(executorResult)}`);
      return executorResult;
    } else if (lambdaEvent.resolveSource === 'EventBus') {
      log.debug('identified event source: event-bus');
      const {
        method,
        payload
      } = lambdaEvent;
      const executorResult = await resolve.eventListener[method](payload);
      log.verbose(`executorResult: ${JSON.stringify(executorResult)}`);
      return executorResult;
    } else if (lambdaEvent.resolveSource === 'EventStore') {
      log.debug('identified event source: event-store');
      const {
        method,
        payload
      } = lambdaEvent;
      const executorResult = await resolve.eventStore[method](payload);
      log.verbose(`executorResult: ${JSON.stringify(executorResult)}`);
      return executorResult;
    } else if (lambdaEvent.resolveSource === 'Scheduler') {
      log.debug('identified event source: cloud scheduler');
      const executorResult = await (0, _schedulerEventHandler.default)(lambdaEvent, resolve);
      log.verbose(`executorResult: ${JSON.stringify(executorResult)}`);
      return executorResult;
    } else if (lambdaEvent.headers != null && lambdaEvent.httpMethod != null) {
      log.debug('identified event source: API gateway');
      log.verbose(JSON.stringify(lambdaEvent.httpMethod, null, 2), JSON.stringify(lambdaEvent.headers, null, 2));
      const executorResult = await (0, _apiGatewayHandler.default)(lambdaEvent, lambdaContext, resolve);
      log.verbose(`executorResult: ${JSON.stringify(executorResult)}`);
      return executorResult;
    } else {
      throw new Error(`abnormal lambda execution on event ${JSON.stringify(lambdaEvent)}`);
    }
  } catch (error) {
    log.error('top-level event handler execution error!');
    await (0, _metrics.putInternalError)(error);

    if (error instanceof Error) {
      log.error('error', error.message);
      log.error('error', error.stack);
    } else {
      log.error(JSON.stringify(error));
    }

    throw error;
  } finally {
    await (0, _disposeResolve.default)(resolve);

    if (process.env.RESOLVE_PERFORMANCE_MONITORING) {
      await (0, _metrics.putDurationMetrics)(lambdaEvent, lambdaContext, coldStart, lambdaRemainingTimeStart);
    }

    coldStart = false;
    log.debug('reSolve framework was disposed');
  }
};

var _default = lambdaWorker;
exports.default = _default;
//# sourceMappingURL=lambda-worker.js.map